import os
import sys
import time
import string
import ID3
import mp3
from status import stat
from data import data
from debug import debug

UnknownTrackTypeError = "This file type is not supported by Cymbaline."

class Track:
    def __init__(self, path, codec):
        self.path   = os.path.join(os.getcwd(), path)
        self.fname  = os.path.basename(path)
        self.mkname()
        self.codec  = codec
        self.secs   = None
        self.mins   = None
        self.total  = None
        self.title  = None
        self.artist = None


    def __str__(self):
        """Print the track.

        artist - title (or just the filename if there's no id3 tags.
        """

        if self.codec == "mp3":
            self.id3()
            if self.artist and self.title:
                return repr("%s - %s" % (self.artist, self.title))
        return repr(self.title) or repr(self.name)


    def mkname(self):
        n = string.join(self.fname.split('.')[:-1], '.')
        while n[0] in string.digits:
            n = n[1:]
        if n[0] in ".)-":
            n = n[1:]
        if n[:3] == " - ": n = n[3:]
        n = n.strip()
        if n.startswith("of"):
            n = n[2:]
        # this is to deal with some names that use foreign characters &
        # terminal sometimes does odd things with those..
        self.name = repr(string.capwords(n.replace('_', ' ')))[1:-1]
        self.name = self.name.replace('\x00','')


    def id3name(self):
        ''' Make name from id3 tags if present. '''
        self.id3()
        self.artist_title = ''
        if self.title: 
            tmp = self.fname.split('.')[0]
            self.name = "%s - %s" % (tmp, self.title)
        if self.artist and self.title:
            self.name = '%s - %s' % (self.artist, self.title)

    def id3(self, reload=0):
        """Get id3 tags and add them as instance attributes."""
        import string
        if (self.title or self.artist) and not reload:
            # tags already loaded; save some time!
            return
        try: 
            id3 = ID3.ID3(self.path)
        except:
            return
        if id3.has_key("TITLE"):
            self.title   = string.capwords(id3["TITLE"])
        if id3.has_key("ARTIST"):
            self.title   = string.capwords(id3["ARTIST"])
        if id3.has_key("ALBUM"):
            self.title   = string.capwords(id3["ALBUM"])
        if id3.has_key("YEAR"):
            self.title   = string.capwords(id3["YEAR"])
        if id3.has_key("COMMENT"):
            self.title   = string.capwords(id3["COMMENT"])
        variables = 'title artist album year'.split()
        valid_chars = string.printable[:95]
        for v in variables:
            # only keep normal printable stuff..
            self.__dict__[v] = ''
            val = self.__dict__[v]
            nval = ''
            for c in val:
                if c in valid_chars:
                    nval += c
            self.__dict__[v] = nval



    def info(self):
        """Load track info into instance variables.

        Loads bitrate, secs, mins, total (time).
        """

        if not self:
            return None
        try:
            if not os.path.exists(self.path):
                return None
        except:
            return None
        if self.secs:
            # save time!
            return None
        self.secs = self.mins = 0
        self.bitrate = 128
        try:
            d = mp3.mp3info(self.path)
        except mp3.BadFile:
            d = None
            #stat.error("Error scanning for mp3 info in " + self.fname)
            #return None
        if d:
            self.secs = d["ss"]
            self.mins = d["mm"]
            self.bitrate = d["bitrate"]
        self.total = int(self.mins)*60 + int(self.secs)
        return 1


    def id3info(self):
        """Display all id3 info."""
        info = []
        if self.codec == "mp3":
            flds = "artist title album year comment".split()
            self.id3()
            for f in flds:
                info.append("%10s : %s" % (f, eval("self."+f)))
        info.append("Score: %s" % self.score())
        import ui
        ui.display(info)


    def score(self):
        from scores import scores
        n = self.fname.lower()
        if scores._dict.has_key(n):
            return scores._dict[n]
        else:
            return None
